# -*- coding: utf-8 -*-

"""Provider for SemanticScholar

Author: G.J.J. van den Burg
License: See LICENSE file
Copyright: 2020, G.J.J. van den Burg

"""

import re
import bs4

from ._base import Provider
from ._info import Informer
from ..exceptions import URLResolutionError
from ..utils import get_page_with_retry, get_content_type_with_retry


class SemanticScholarInformer(Informer):

    meta_date_key = "citation_publication_date"

    def _format_authors(self, soup_authors):
        return super()._format_authors(soup_authors, sep=" ", idx=-1)


class SemanticScholar(Provider):

    re_abs = (
        "https?:\/\/www.semanticscholar.org/paper/[A-Za-z0-9%\-]+/[0-9a-f]{40}"
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.informer = SemanticScholarInformer()

    def get_abs_pdf_urls(self, url):
        """ Get the pdf and abstract urls from a SemanticScholar url """
        if re.match(self.re_abs, url):
            abs_url = url
            pdf_url = self._get_pdf_url(abs_url)
        else:
            raise URLResolutionError("SemanticScholar", url)
        return abs_url, pdf_url

    def _get_pdf_url(self, url):
        page = get_page_with_retry(url)
        soup = bs4.BeautifulSoup(page, "html.parser")

        # First try to get the direct url to the PDF file from the HTML
        a = soup.find(
            "a",
            {
                "data-selenium-selector": "paper-link",
                "data-heap-direct-pdf-link": "true",
            },
        )
        if a:
            return a["href"]

        # Next try to get the url from the metadata (not always a pdf)
        meta = soup.find_all("meta", {"name": "citation_pdf_url"})
        if not meta:
            raise URLResolutionError(
                "SemanticScholar", url, reason="Page has no url to PDF file"
            )
        pdf_url = meta[0]["content"]

        # Check the content type to check that the data will be a pdf
        content_type = get_content_type_with_retry(pdf_url)
        if content_type is None:
            raise URLResolutionError(
                "SemanticScholar",
                url,
                reason="Can't determine content type for pdf file",
            )
        if not content_type == "application/pdf":
            raise URLResolutionError(
                "SemanticScholar",
                url,
                reason="PDF url on SemanticScholar doesn't point to a pdf file",
            )
        return pdf_url

    def validate(src):
        return re.match(SemanticScholar.re_abs, src)



# -*- coding: utf-8 -*-

"""Provider for ACL

Author: G.J.J. van den Burg
License: See LICENSE file
Copyright: 2021, G.J.J. van den Burg

"""

import re

from ._base import Provider
from ._info import Informer
from ..exceptions import URLResolutionError


class ACLInformer(Informer):

    meta_date_key = "citation_publication_date"

    def _format_authors(self, soup_authors):
        return super()._format_authors(soup_authors, sep=" ", idx=-1)


class ACL(Provider):

    re_abs_1 = "^https://www.aclweb.org/anthology/(?P<key>[0-9a-zA-Z\.\-]+)"
    re_abs_2 = "^https://(www.)?aclanthology.org/(?P<key>[0-9a-zA-Z\.\-]+)"
    re_pdf_1 = "^https://www.aclweb.org/anthology/(?P<key>[0-9a-zA-Z\.\-]*?)(v\d+)?.pdf"
    re_pdf_2 = "^https://(www.)?aclanthology.org/(?P<key>[0-9a-zA-Z\.\-]*?)(v\d+)?.pdf"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.informer = ACLInformer()

    def get_abs_pdf_urls(self, url):
        m = re.match(self.re_pdf_1, url)
        if m:
            pdf_url = url
            abs_url = f"https://www.aclweb.org/anthology/{m['key']}"
            return abs_url, pdf_url

        m = re.match(self.re_pdf_2, url)
        if m:
            pdf_url = url
            abs_url = f"https://www.aclanthology.org/{m['key']}"
            return abs_url, pdf_url

        m = re.match(self.re_abs_1, url)
        if m:
            abs_url = url
            pdf_url = f"https://www.aclweb.org/anthology/{m['key']}.pdf"
            return abs_url, pdf_url

        m = re.match(self.re_abs_2, url)
        if m:
            abs_url = url
            pdf_url = f"https://www.aclanthology.org/{m['key']}.pdf"
            return abs_url, pdf_url

        raise URLResolutionError("ACL", url)

    def validate(src):
        return (
            re.match(ACL.re_pdf_1, src)
            or re.match(ACL.re_pdf_2, src)
            or re.match(ACL.re_abs_1, src)
            or re.match(ACL.re_abs_2, src)
        )



# -*- coding: utf-8 -*-

"""Provider for IACR's eprints

Author: G.J.J. van den Burg
License: See LICENSE file
Copyright: 2019, G.J.J. van den Burg

"""

import bs4
import os
import re
import urllib.parse

from ._info import Informer
from ._base import Provider
from ..exceptions import URLResolutionError
from ..log import Logger
from ..utils import get_page_with_retry

logger = Logger()


class IACRInformer(Informer):
    def get_title(self, soup):
        title = soup.find_all("title")
        if not title:
            logger.warning(
                "Couldn't determine title information, maybe provide the desired filename using '--filename'?"
            )
            return ""
        return title[0].get_text().split("-", maxsplit=1)[-1]

    def get_authors(self, soup):
        i = soup.find_all("i")
        if not i:
            logger.warning(
                "Couldn't determine author information, maybe provide the desired filename using '--filename'?"
            )
            return ""
        authors = i[0].get_text()
        authors = authors.replace("  ", " ")
        authors = authors.split(" and ")
        return self._format_authors(authors, sep=" ", idx=-1)

    def get_year(self, soup):
        h2 = soup.find_all("h2")
        if not h2:
            logger.warning(
                "Couldn't determine year information, maybe provide the desired filename using '--filename'?"
            )
            return ""
        text = h2[0].get_text()
        report = text.split(":", maxsplit=1)[-1]
        year_num = report.strip().split(" ")[1]
        year = year_num.split("/")[0]
        return year


class IACR(Provider):

    re_abs = "https?://eprint.iacr.org/\d{4}/\d+$"
    re_pdf = "https?://eprint.iacr.org/\d{4}/\d+\.pdf$"
    re_ps = "https?://eprint.iacr.org/\d{4}/\d+\.ps$"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.informer = IACRInformer()

    def _get_doc_url(self, abs_url):
        page = get_page_with_retry(abs_url)
        soup = bs4.BeautifulSoup(page, "html.parser")

        bb = soup.find_all("b")
        b = next((b for b in bb if "Available format" in b.get_text()), None)
        if b is None:
            # Fallback
            return abs_url + ".pdf"
        aa = b.find_next_siblings("a")
        a = next((a for a in aa if "PDF" in a.get_text()), None)
        if not a is None:
            return urllib.parse.urljoin(abs_url, a.get("href"))
        a = next((a for a in aa if "Postscript (PS)" in a.get_text()), None)
        if not a is None:
            return urllib.parse.urljoin(abs_url, a.get("href"))
        # Fallback
        return abs_url + ".pdf"

    def get_abs_pdf_urls(self, url):
        if re.match(self.re_abs, url):
            abs_url = url
            pdf_url = self._get_doc_url(url)
        elif re.match(self.re_pdf, url):
            abs_url = url[: -len(".pdf")]
            pdf_url = url
        elif re.match(self.re_ps, url):
            abs_url = url[: -len(".ps")]
            pdf_url = url
        else:
            raise URLResolutionError("IACR", url)
        return abs_url, pdf_url

    def retrieve_pdf(self, pdf_url, filename):
        # Bit hacky, can consider adding first-class PS support
        tmpfilename = os.path.splitext(filename)[0] + "-tmp.pdf"
        super().retrieve_pdf(pdf_url, tmpfilename)
        self.rewrite_pdf(tmpfilename, out_pdf=filename)

    def validate(src):
        return re.match(IACR.re_abs, src) or re.match(IACR.re_pdf, src)
